*+
*+
*+    Source Module => SHOWTIME.PRG
*+
*+    Functions: Procedure showtime()
*+               Procedure clockgears()
*+               Procedure putclock()
*+               Procedure getclock()
*+
*+    Reformatted by Click! 1.11 on Oct-11-1997 at  3:53 am
*+
*+

#include "COMMON.CH"

static sClock     := { 0, 0, .f., 'w/n', .f., .t., .f. }
static ClockStack := {}

// ShowTime() with parameters turns on the clock.
// ShowTime() with no parameters turns off the clock.

// It requires the NanForum library FT_ONTICK() function. (PUBLIC DOMAIN)

// Donated to the PUBLIC DOMAIN by Phil Barnett November 29, 1996
// Revised                      by Phil Barnett March    20, 1997

*+
*+
*+    Procedure showtime()
*+
*+    Called from ( showtime.prg )   1 - procedure showtime()
*+                                   1 - procedure putclock()
*+                                   1 - procedure getclock()
*+
*+
*+
procedure showtime( nRow, nCol, lShowSeconds, cColor, lMilTime, lAmPm )

begin sequence

   if isnil( nRow ) .and. ;
             isnil( nCol ) .and. ;
             isnil( lshowseconds ) .and. ;
             isnil( cColor ) .and. ;
             isnil( lmiltime ) .and. ;
             isnil( lAmPm )

      sClock[ 7 ] := .f.

      break

   endif

   // any parameter that is not passed in, defaults to the previous value

   default nRow to sClock[ 1 ]
   default nCol to sClock[ 2 ]
   default lShowSeconds to sClock[ 3 ]
   default cColor to sClock[ 4 ]
   default lMilTime to sClock[ 5 ]
   default lAmPm to sClock[ 6 ]

   sClock[ 1 ] := nRow
   sClock[ 2 ] := nCol
   sClock[ 3 ] := lShowSeconds
   sClock[ 4 ] := alltrim( cColor )
   sClock[ 5 ] := lMilTime
   sClock[ 6 ] := lAmPm
   sClock[ 7 ] := .t.

   FT_ONTICK( { || clockgears() }, iif( sClock[ 3 ], 2, 18 ) )

end sequence

return

*+
*+
*+    Procedure clockgears()
*+
*+    Called from ( showtime.prg )   1 - procedure showtime()
*+
*+
*+
procedure clockgears()

// clockgears() gets called by ft_ontick() to update the display

local nRow
local nCol
local nCurState
local cTime
local nHours
local lAM

if sClock[ 7 ]

   nRow      := row()
   nCol      := col()
   nCurState := set( _SET_CURSOR, 0 )
   cTime     := time()
   nHours    := val( left( cTime, 2 ) )
   lAM       := nHours < 12

   if !sClock[ 3 ]
      cTime := left( cTime, 5 )
   endif

   if !sClock[ 5 ]
      if nHours >= 12
         nHours -= 12
      endif
      if nHours = 0
         nHours := 12
      endif
      cTime := str( nHours, 2 ) + substr( cTime, 3 )
   endif

   if sClock[ 6 ]
      cTime += if( lAM, 'a', 'p' )
   endif

   @ sClock[  1 ], sClock[ 2 ] say cTime color sClock[ 4 ]        

   devpos( nRow, ncol )
   set( _SET_CURSOR, nCurState )

endif

return

*+
*+
*+    Procedure putclock()
*+
*+
*+
procedure putclock( lTurnOff )

// This function puts the contents of the current
// Clock Array onto the ClockStack array.

// PutClock() defaults to turning the clock off.
// Pass .f. to keep the clock running.

aadd( ClockStack, aclone( sClock ) )

default lTurnOff to .t.

if lTurnOff
   showtime()
endif

return

*+
*+
*+    Procedure getclock()
*+
*+
*+
procedure getclock()

// If there is an entry in the stack, restore it to the sClock array

local StackLen     := len( ClockStack )
local nRow
local nCol
local lShowSeconds
local cColor
local lMilTime
local lAmPm

if StackLen > 0

   sClock := aclone( ClockStack[ StackLen ] )
   asize( ClockStack, StackLen - 1 )

   nRow         := sClock[ 1 ]
   nCol         := sClock[ 2 ]
   lShowSeconds := sClock[ 3 ]
   cColor       := sClock[ 4 ]
   lMilTime     := sClock[ 5 ]
   lAmPm        := sClock[ 6 ]

   showtime( nRow, nCol, lShowSeconds, cColor, lMilTime, lAmPm )

endif

return

*+ EOF: SHOWTIME.PRG
